﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Android.App;
using Android.Content;
using Android.Graphics;
using Android.Net;
using Android.OS;
using Android.Runtime;
using Android.Support.V4.App;
using Android.Views;
using Android.Widget;

namespace App1
{

    [Service]
	public class CustomService : Service
	{
		private Handler handler;
		private Action runnable;
		private bool isStarted;
		private int DELAY_BETWEEN_LOG_MESSAGES = 5000;
		private int NOTIFICATION_SERVICE_ID = 1001;
		private int NOTIFICATION_AlARM_ID = 1002;
		public override void OnCreate()
		{
			base.OnCreate();

			handler = new Handler();

			runnable = new Action(() =>
			{
				if (isStarted)
				{
					IntentFilter filter = new IntentFilter();
					filter.AddAction(ConnectivityManager.ConnectivityAction);
					RegisterReceiver(new MyReceiver(), filter);
				}
			});
		}

		public override StartCommandResult OnStartCommand(Intent intent, StartCommandFlags flags, int startId)
		{
			if (isStarted)
			{
				// service is already started
			}
			else
			{
				DispatchNotificationThatServiceIsRunning();

				handler.PostDelayed(runnable, DELAY_BETWEEN_LOG_MESSAGES);
				isStarted = true;
			}
			return StartCommandResult.Sticky;
		}

		public override void OnTaskRemoved(Intent rootIntent)
		{
			//base.OnTaskRemoved(rootIntent);
		}

		public override IBinder OnBind(Intent intent)
		{
			return null;
		}

		public override void OnDestroy()
		{
			// Stop the handler.
			handler.RemoveCallbacks(runnable);

			// Remove the notification from the status bar.
			var notificationManager = (NotificationManager)GetSystemService(NotificationService);
			notificationManager.Cancel(NOTIFICATION_SERVICE_ID);

			isStarted = false;
			base.OnDestroy();
		}

		//start a foreground notification to keep alive
		private void DispatchNotificationThatServiceIsRunning()
		{
			var notificationManager = (NotificationManager)GetSystemService(NotificationService);

			if (Build.VERSION.SdkInt > BuildVersionCodes.O)
			{
				NotificationCompat.Builder builder = new NotificationCompat.Builder(this, "default")
				   .SetAutoCancel(false)
				   .SetSmallIcon(Resource.Drawable.img)
				   .SetContentTitle("Mobile")
				   .SetContentText("My service started");

				var chan = new NotificationChannel("default", GetString(Resource.String.abc_action_bar_home_description), NotificationImportance.Default);
				chan.LightColor = Color.Green;
				chan.LockscreenVisibility = NotificationVisibility.Private;

				notificationManager.CreateNotificationChannel(chan);

				StartForeground(NOTIFICATION_SERVICE_ID, builder.Build());
			}
			else
			{
				NotificationCompat.Builder builder = new NotificationCompat.Builder(this)
				   .SetDefaults((int)NotificationDefaults.All)
				   .SetSmallIcon(Resource.Drawable.img)
				   .SetVibrate(new long[] { 100, 200, 300, 400, 500, 400, 300, 200, 400 })
				   .SetSound(null)
				   .SetPriority(NotificationCompat.PriorityDefault)
				   .SetAutoCancel(false)
				   .SetContentTitle("Mobile")
				   .SetContentText("My service started")
				   .SetOngoing(true);

				StartForeground(NOTIFICATION_SERVICE_ID, builder.Build());
			}
		}

		private void DispatchNotificationThatAlarmIsGenerated(string message)
		{
			var intent = new Intent(this, typeof(MainActivity));
			intent.AddFlags(ActivityFlags.ClearTop);
			var pendingIntent = PendingIntent.GetActivity(this, 0, intent, PendingIntentFlags.OneShot);

			var notificationManager = (NotificationManager)GetSystemService(NotificationService);

			if (Build.VERSION.SdkInt > BuildVersionCodes.O)
			{
				NotificationCompat.Builder notificationBuilder = new NotificationCompat.Builder(this, "default")
				.SetContentTitle("Alarm")
				.SetContentText(message)
				.SetSmallIcon(Resource.Drawable.img)
				.SetAutoCancel(true);

				var chan = new NotificationChannel("default", GetString(Resource.String.abc_action_bar_home_description), NotificationImportance.Default);
				chan.LightColor = Color.Green;
				chan.LockscreenVisibility = NotificationVisibility.Private;

				notificationManager.CreateNotificationChannel(chan);
				notificationManager.Notify(NOTIFICATION_AlARM_ID, notificationBuilder.Build());
			}
			else
			{
				NotificationCompat.Builder notificationBuilder = new NotificationCompat.Builder(this)
				   .SetDefaults((int)NotificationDefaults.All)
				   .SetSmallIcon(Resource.Drawable.img)
				   .SetVibrate(new long[] { 100, 200, 300, 400, 500, 400, 300, 200, 400 })
				   .SetSound(null)
				   .SetPriority(NotificationCompat.PriorityDefault)
				   .SetAutoCancel(false)
				   .SetContentTitle("Mobile")
				   .SetContentText("My service started")
				   .SetOngoing(true);

				notificationManager.Notify(NOTIFICATION_AlARM_ID, notificationBuilder.Build());
			}
		}
	}
}